/*****************************************************************************
 * $Id: ix-relink.c,v 1.1 2005/08/28 18:22:43 killabyte Exp $
 *
 * Here is implemented the function '_pdi_arch_relink()' for Irix which is
 * responsible at low level to install a relink interposition.
 *
 * ---------------------------------------------------------------------------
 * pDI-Tools - portable Dynamic Instrumentation Tools
 *   (C) 2004, 2005 Gerardo Garca Pea
 *   Programmed by Gerardo Garca Pea - Inspired on CEPBA DItools
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 *   USA
 *
 *****************************************************************************/

#include<config.h>
#include<ebeif.h>
#include<log.h>
#include<irix/elfops.h>

/*****************************************************************************
 * int _pdi_irix_relink(PDI_ELFOBJ *eo, PDI_INTERCEPT *i)
 *
 * Description:
 *   Relinks calls from object 'eo' to some function to another function in a
 *   backend. The interposition parameters are in 'i'.
 *
 * Parameters:
 *   eo - relinked object
 *   i  - relink parameters
 *
 * Returns:
 *   0 if all ok, a negative value otherwise.
 *
 *****************************************************************************/

int _pdi_irix_relink(PDI_ELFOBJ *eo, PDI_INTERCEPT *i)
{
  unsigned int symndx;
  ElfW(Sym) *symptr;

  /* Search symbol in the symtab */
  if((symptr = _pdi_irix_getSymbolTableEntry(eo->o_arch,
                                             i->i_symbol,
                                             &symndx)) == NULL)
  {
    _pdi_error(THIS, "Cannot find symbol '%s' in '%s'.",
               i->i_symbol, _pdi_ebe_getObjectName(eo));
    return -1;
  }

  /* Execute relink */
  if(_pdi_irix_alterPlt(eo,
                        symndx,
                        i->i_wrapper,
                        &(i->i_arch->u.relink.undo_data)))
  {
    _pdi_error(THIS, "Cannot modify GOT/PLT entry of symbol '%s::%s()'.",
               _pdi_ebe_getObjectName(eo), i->i_symbol);
    return -1;
  }

  return 0;
}

/*****************************************************************************
 * int _pdi_irix_undoRelink(PDI_ELFOBJ *eo, PDI_INTERCEPT *i)
 *
 * Description:
 *   Undo relink interposition 'i' in object 'eo'.
 *
 * Parameters:
 *   eo - relinked object
 *   i  - relink parameters
 *
 * Returns:
 *   0 if all ok, a negative value otherwise.
 *
 *****************************************************************************/

int _pdi_irix_undoRelink(PDI_ELFOBJ *eo, PDI_INTERCEPT *i)
{
  _pdi_irix_restorePlt(&(i->i_arch->u.relink.undo_data));

  return 0;
}

